/*
 *
 * Copyright (c) 2001-2018 泛微软件.
 * 泛微协同商务系统,版权所有.
 *
 */
package com.cloudstore.dev.api.util;

import cn.hutool.core.lang.Assert;
import com.cloudstore.dev.api.bean.OtherSystemInfo;
import com.cloudstore.dev.api.service.SessionFilter;
import com.cloudstore.eccom.common.WeaIndexManager;
import com.wbi.core.biz.connect.engine.Connect;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import weaver.conn.ConnectionPool;
import weaver.conn.RecordSet;
import weaver.file.Prop;
import weaver.general.BaseBean;
import weaver.general.PasswordUtil;
import weaver.general.Util;
import weaver.rsa.security.Base64;
import weaver.rsa.security.RSA;

import javax.servlet.http.HttpServletRequest;
import java.security.KeyPair;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * AuthManager 简化版本的认证管理器
 *
 * @author AndyZhang
 * TODO
 * @version 1.0
 */
public class AuthManager {

    private static final Log logger = LogFactory.getLog(AuthManager.class);

    public static final String secret = "secret";

    public static final String appid = "appid";

    public static final String cpk = "cpk";

    public static final String loginid = "loginid";

    public static final String password = "pwd";
    //5分钟超时
    public static final String access_token_time = "access_token_time";

    public static final String access_token = "access_token";

    /**
     * 检查管理员帐号密码
     *
     * @param loginid
     * @param password
     * @return
     */
    public static boolean checkAdminPassword(String loginid, String password) {
        return PasswordUtil.checkPasswordByLoginid(loginid, password, true);
    }

    /**
     * 检查帐号密码
     *
     * @param loginid
     * @param password
     * @return
     */
    public static boolean checkPassword(String loginid, String password) {
        boolean isAdmin = false;
        if ("sysadmin".equals(loginid)) {
            isAdmin = true;
        }
        return PasswordUtil.checkPasswordByLoginid(loginid, password, isAdmin);
    }

    /**
     * 检查是否有注册信息
     *
     * @param appid
     * @return
     */
    public static boolean hasRegistData(String appid) {
        //if (StringUtils.isBlank(appid)) return false;
        //if (ConnectionPool.getInstance().isInitiating()) return false;//初始化里，默认查询存在
        RecordSet rs = new RecordSet();
        rs.executeQuery("select * from ECOLOGY_BIZ_EC where APPID=?", appid);
        return rs.next();
    }

    /**
     * 更新安全码
     *
     * @param appid
     * @param cpk
     * @return
     */
    public static String updateSecret(String appid, String cpk, KeyPair keyPair) {
        if (Objects.nonNull(keyPair)){
            RecordSet rs = new RecordSet();
            String secret = WeaIndexManager.getGuid();
            String privateKey = Base64.encodeBase64String(keyPair.getPrivate().getEncoded());
            String publicKey = Base64.encodeBase64String(keyPair.getPublic().getEncoded());
            rs.executeUpdate("update ECOLOGY_BIZ_EC set RSA_PUB=?,SECRIT=?,RSA_PRV=?,SERVER_PUBLIC_KEY=? where APPID=?", cpk, secret,privateKey,publicKey,appid);
            return secret;
        }
        return null;
    }

    /**
     * 创建Token
     *
     * @param userid
     * @param secret
     * @return
     */
    public static String createToken(String userid, String secret) {
        Date dt = new Date();
        String token = Util_Security.decoded_des(userid + "=" + dt.getTime() + "=" + secret);
        return token;
    }

    /**
     * 解密数据
     *
     * @param data
     * @return
     */
    public static String decrypt(String data) {
        RSA rsa = new RSA();
        String secret = rsa.decrypt(null, data, true);
        return secret;
    }

    /**
     * 加密数据
     *
     * @param data
     * @param cpk
     * @return
     */
    public static String encrypt(String data, String cpk) {
        RSA rsa = new RSA();
        String secret = rsa.encrypt(null, data, null, "UTF-8", cpk, false);
        return secret;
    }


    /**
     * 获取注册信息
     * @param appid
     * @return
     */
    public static OtherSystemInfo getAppInfo(String appid) {
        OtherSystemInfo info = new OtherSystemInfo();
        if(StringUtils.isNotBlank(appid)){
            RecordSet rs = new RecordSet();
            rs.executeQuery("select * from ECOLOGY_BIZ_EC where APPID=?", appid);
            if (rs.next()) {
                info.setAppid(appid);
                info.setSecret(rs.getString("SECRIT"));
                info.setClientPublicKey(rs.getString("RSA_PUB"));
                info.setServerPrivateKey(rs.getString("RSA_PRV"));
                info.setServerPublicKey(rs.getString("SERVER_PUBLIC_KEY"));
            }
        }
        return info;
    }


    public static boolean allowIp(HttpServletRequest request){
        BaseBean baseBean = new BaseBean();
        String ips = baseBean.getPropValue("weaver_rest_token","allowIp");
        if (StringUtils.isBlank(ips)) return true;
        String ip = Util.getIpAddr(request);
        for (String ipTemp : StringUtils.split(ips,",")){
            if (ip.equalsIgnoreCase(ipTemp)){
                return true;
            }
        }
        return false;
    }

    /**
     *
     *  @param systemName: 对接业务系统名称
     */
    public static String generateAppid(String systemName){
        return generateAppid(systemName, Collections.EMPTY_LIST);
    }

    public static String generateAppid(String systemName, List<String> allowIps){
        Assert.notEmpty(systemName,"systemName can not is empty.");
        if (!checkEnabled()) forceEnable();
        String uuid = WeaIndexManager.getGuid(16);
        RecordSet rs = new RecordSet();
        rs.executeQuery("select appid from ecology_biz_ec where name =?",systemName);
        if (rs.next()){
            return rs.getString("appid");
        }
        rs.executeUpdate("insert into ecology_biz_ec(id,appid,name) values(?,?,?)",uuid.hashCode(),uuid,systemName);
        return uuid;
    }

    /**
     * 检查当前环境是否已开启Rest Token方案
     */
    public static synchronized boolean checkEnabled(){
        BaseBean bean = new BaseBean();
        String urls = bean.getPropValue("weaver_session_filter","unchecksessionurl");
        if (StringUtils.isBlank(urls)) return false;
        return urls.indexOf("/api/ec/dev/auth")>=0;
    }

    /**
     * 强制开启Rest Token认证方案
     */
    public static synchronized void forceEnable(){
        String apiUrl = "/api/ec/dev/auth;";
        String urls = Prop.getPropValue("weaver_session_filter","unchecksessionurl");
        if (StringUtils.isNotBlank(urls)&&urls.endsWith(";")){
            urls += apiUrl;
            SessionFilter.appendUncheckSessionUrl(apiUrl);
        } else {
            urls += ";"+apiUrl;
            SessionFilter.appendUncheckSessionUrl(";"+apiUrl);
        }
        Prop.setPropValue("weaver_session_filter","unchecksessionurl",urls);
    }

}
