package weaver.file;

import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.AutoDetectParser;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.parser.Parser;
import org.apache.tika.sax.BodyContentHandler;
import org.xml.sax.ContentHandler;
import weaver.general.BaseBean;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.CopyOnWriteArrayList;

public class MineType {

    private static List mimeTypeWhiteList = new CopyOnWriteArrayList<String>();

    static{
        init();
    }

    private String mime;

    public String getMime() {
        return mime;
    }

    private static void init()
    {
        mimeTypeWhiteList.add("image/bmp");
        mimeTypeWhiteList.add("application/msword");
        mimeTypeWhiteList.add("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
        mimeTypeWhiteList.add("image/gif");
        mimeTypeWhiteList.add("image/vnd.microsoft.ico");
        mimeTypeWhiteList.add("image/jpeg");
        mimeTypeWhiteList.add("image/png");
        mimeTypeWhiteList.add("image/heic");
		mimeTypeWhiteList.add("image/x-portable-graymap");		
        mimeTypeWhiteList.add("application/pdf");
        mimeTypeWhiteList.add("application/vnd.ms-powerpoint");
        mimeTypeWhiteList.add("application/vnd.openxmlformats-officedocument.presentationml.presentation");
        mimeTypeWhiteList.add("application/x-rar-compressed");
        mimeTypeWhiteList.add("application/x-rar-compressed; version=5");
		mimeTypeWhiteList.add("application/x-rar-compressed; version=4");
        mimeTypeWhiteList.add("application/vnd.ms-excel");
        mimeTypeWhiteList.add("application/zip");
        mimeTypeWhiteList.add("application/rtf");
        mimeTypeWhiteList.add("application/vnd.ms-outlook");
        mimeTypeWhiteList.add("application/vnd.ms-project");
        mimeTypeWhiteList.add("application/x-gzip");
        mimeTypeWhiteList.add("application/x-gtar");
        mimeTypeWhiteList.add("application/gzip");
        mimeTypeWhiteList.add("application/x-internet-signup");
        mimeTypeWhiteList.add("application/x-shockwave-flash");
        mimeTypeWhiteList.add("application/x-tika-ooxml");
        mimeTypeWhiteList.add("application/x-tika-msoffice");
        mimeTypeWhiteList.add("application/xml");

        mimeTypeWhiteList.add("application/x-tar");
        mimeTypeWhiteList.add("application/x-troff");
        mimeTypeWhiteList.add("application/x-x509-ca-cert");
        mimeTypeWhiteList.add("audio/basic");
        mimeTypeWhiteList.add("audio/mid");
        mimeTypeWhiteList.add("audio/mpeg");
        mimeTypeWhiteList.add("audio/x-wav");
        mimeTypeWhiteList.add("image/bmp");
        mimeTypeWhiteList.add("image/svg+xml");

        mimeTypeWhiteList.add("image/vnd.dwg");
        mimeTypeWhiteList.add("image/vnd.dxf; format=ascii");

        mimeTypeWhiteList.add("image/tiff");
        mimeTypeWhiteList.add("image/x-icon");
		mimeTypeWhiteList.add("text/html");
        mimeTypeWhiteList.add("text/plain");
        mimeTypeWhiteList.add("video/mpeg");
        mimeTypeWhiteList.add("video/quicktime");
		mimeTypeWhiteList.add("video/mp4");
        mimeTypeWhiteList.add("video/x-msvideo");
        mimeTypeWhiteList.add("video/x-sgi-movie");
		mimeTypeWhiteList.add("video/x-flv");
        mimeTypeWhiteList.add("video/x-ms-wmv");
        mimeTypeWhiteList.add("application/x-font-ttf");
        mimeTypeWhiteList.add("application/x-msaccess");
        mimeTypeWhiteList.add("application/octet-stream");
		mimeTypeWhiteList.add("application/x-7z-compressed");
		mimeTypeWhiteList.add("application/java-vm");
		mimeTypeWhiteList.add("message/rfc822");
        mimeTypeWhiteList.add("audio/vnd.wave");
		mimeTypeWhiteList.add("application/x-msdownload");
		mimeTypeWhiteList.add("application/x-sqlite3");
		mimeTypeWhiteList.add("application/vnd.ms-cab-compressed");
        mimeTypeWhiteList.add("application/vnd.ms-spreadsheetml");
        mimeTypeWhiteList.add("application/gzip");
        mimeTypeWhiteList.add("image/webp");
		mimeTypeWhiteList.add("application/vnd.ms-word2006ml");
		mimeTypeWhiteList.add("application/vnd.ms-wordml");
		mimeTypeWhiteList.add("image/vnd.adobe.photoshop");
		mimeTypeWhiteList.add("application/java-vm");
		mimeTypeWhiteList.add("application/postscript");
		mimeTypeWhiteList.add("application/vnd.ms-excel.sheet.4");
		mimeTypeWhiteList.add("application/x-x509-key; format=der");		
		mimeTypeWhiteList.add("text/x-matlab");
		mimeTypeWhiteList.add("model/vnd.dwf; version=6");
        BaseBean bb = new BaseBean();
        String fileTypes = bb.getPropValue("weaver_allow_mime","mime");
        if(fileTypes!=null&&!fileTypes.equals("")){
            String[] fileTypeArr = fileTypes.split(",");
            for(int i=0;i<fileTypeArr.length;i++){
                try{
                    String fileType = fileTypeArr[i].toLowerCase().trim();
                    if(fileType!=null && !"".equals(fileType) && !mimeTypeWhiteList.contains(fileType)){
                        mimeTypeWhiteList.add(fileType);
                    }
                }catch(Exception e){
                    bb.writeLog(e);
                }
            }
        }
    }

    public boolean checkFileType(InputStream inputStream){
        try {
            // MultipartFile to File
            Parser parser= new AutoDetectParser();
            int writerHandler =-1;
            ContentHandler contentHandler= new BodyContentHandler(writerHandler);

            Metadata metadata= new Metadata();

            parser.parse(inputStream, contentHandler, metadata, new ParseContext());

            String mimeType = metadata.get(Metadata.CONTENT_TYPE);
            mime = mimeType;
            boolean anyMatch = mimeTypeWhiteList.contains(mimeType.toLowerCase(Locale.US));
            if(!anyMatch){
                new BaseBean().writeLog("文件类型异常，检测到的文件类型为："+mimeType);
            }
            return anyMatch;
        }catch (Exception e) {
            new BaseBean().writeLog(e);
        }

        return false;
    }

    public static void main(String[] args) throws FileNotFoundException {
        File file = new File("F:\\【1617715】文档数量显示问题-操作手册.doc");
        System.out.println(new MineType().checkFileType(new FileInputStream(file)));
    }


}
