package weaver.integration.util;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.CookieStore;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.CharArrayBuffer;
import org.apache.http.util.EntityUtils;
import weaver.file.Prop;
import weaver.general.GCONST;
import weaver.integration.logging.Logger;
import weaver.integration.logging.LoggerFactory;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.nio.charset.Charset;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.UUID;

/**
 * Created by crazyDream on 2019/4/23.
 */
public class NodeUtil {
    public static String SCHEDULE_REST_URL = "";

    public static ArrayList<String> realIp=new ArrayList();

    public String postJSON(String restUrl, String params) {
        logger.info("=========post url:" + restUrl + ",params:" + params);
        HttpPost post = null;
        String respStr = "";
        try {
            org.apache.http.client.HttpClient httpClient = getHttpClientSSL();

            String apiURL = restUrl;

            post = new HttpPost(apiURL);
            // 构造消息头
            post.setHeader("Content-type", "application/json; charset=utf-8");
            post.setHeader("Connection", "Close");
            String sessionId = getSessionId();
            post.setHeader("SessionId", sessionId);

            StringEntity entity = new StringEntity(params, Charset.forName("UTF-8"));
            entity.setContentEncoding("UTF-8");
            entity.setContentType("application/json");
            post.setEntity(entity);

            HttpResponse response = httpClient.execute(post);
            respStr = entityToString(response.getEntity());
            logger.info("===============resp:\n" + respStr);


            // 检验返回码
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != HttpStatus.SC_OK) {
                logger.info("====================请求出错: " + statusCode);
            } else {
                int retCode = 0;
                String sessendId = "";
                // 返回码中包含retCode及会话Id
                for (Header header : response.getAllHeaders()) {
                    if (header.getName().equals("retcode")) {
                        retCode = Integer.parseInt(header.getValue());
                    }
                    if (header.getName().equals("SessionId")) {
                        sessendId = header.getValue();
                    }
                }

                if (200 != retCode) {
                    logger.info("==============================error return code,  sessionId: " + sessendId + "\t" + "retCode: " + retCode);
                } else {
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("post error occured!!" + e.getMessage());
        } finally {
            if (post != null) {
                try {
                    post.releaseConnection();
                    //Thread.sleep(500);
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.error(e.getMessage());
                }
            }
        }
        return respStr;
    }


    public static String getMasterNodeScheduleRestApiUrl(HttpServletRequest request) {
        String url = "";
        String mainControlIp = "";
        Prop prop = Prop.getInstance();
        mainControlIp = prop.getPropValue(GCONST.getConfigFile(), "MainControlIP");
        if (!"".equals(mainControlIp)) {
            url = request.getScheme() + "://" + mainControlIp + ":" + request.getServerPort() + "/schedule/rest";
        }
        logger.info("=============getMasterNodeScheduleRestApiUrl:" + url);
        SCHEDULE_REST_URL = url;
        return url;
    }

    /**
     * 是否是主节点(目前计划任务的执行逻辑只在主节点上)
     *
     * @return
     */
    public static boolean isMasterNode() {
        String mainControlIp = "";
        ArrayList<String> hostIps = getRealIp();
        Prop prop = Prop.getInstance();
        mainControlIp = prop.getPropValue(GCONST.getConfigFile(), "MainControlIP");
        if (hostIps == null || hostIps.size() == 0) {
            logger.info("System Init Error:Cannot get local Ip address,This may cause scripts or Timed task  not run! ");
        } else {
            logger.info("System Init Message:mainControlIp=" + mainControlIp + " localIp:" + hostIps.toString());
        }
        if ((!"".equals(mainControlIp) && hostIps.contains(mainControlIp)) || "".equals(mainControlIp)) {
            return true;
        }
        return false;
    }

    /**
     * 获取本机IP列表（包含内网ip，外网ip），存储在集合中<br>
     *
     * @return
     * @throws SocketException
     */
    public static ArrayList<String> getRealIp() {
        if (realIp!=null &&!realIp.isEmpty()) {
            return realIp;
        }
        String localip = null; // 本地IP
        String netip = null;  // 外网IP
        ArrayList<String> ips = new ArrayList<String>();//用于存储所有获取到的本机IP

        Enumeration<NetworkInterface> netInterfaces = null;
        try {
            netInterfaces = NetworkInterface.getNetworkInterfaces();
        } catch (SocketException e) {
            e.printStackTrace();
        } //返回此机器上的所有网络接口，如果所有网络接口都被禁止 则返回null
        InetAddress ip = null;
        while (netInterfaces.hasMoreElements()) { // 遍历多网卡
            NetworkInterface ni = netInterfaces.nextElement();
            Enumeration<InetAddress> address = ni.getInetAddresses(); //获取所有的inetaddresses或绑定到该网络接口的子集枚举对象
            while (address.hasMoreElements()) {
                ip = address.nextElement();
                if (!ip.isSiteLocalAddress() && !ip.isLoopbackAddress() && ip.getHostAddress().indexOf(":") == -1) {// 外网IP(三个条件分别为 不为局域网IP，不为环路IP,不为ipv6 IP)
                    netip = ip.getHostAddress();
                    if (netip != null && !"".equals(netip)) {
                        ips.add(netip);
                    }
                } else if (ip.isSiteLocalAddress() && !ip.isLoopbackAddress() && ip.getHostAddress().indexOf(":") == -1) {// 内网IP(三个条件分别为 是局域网IP，不是环路IP，不是ipv6 Ip)
                    localip = ip.getHostAddress();
                    ips.add(localip);
                }
            }
        }
        realIp = ips;
        return ips;
    }

    /**
     * 绕过验证
     *
     * @return
     * @throws NoSuchAlgorithmException
     * @throws KeyManagementException
     */
    public SSLContext createIgnoreVerifySSL() throws NoSuchAlgorithmException, KeyManagementException {
        SSLContext sc = SSLContext.getInstance("SSLv3");

        // 实现一个X509TrustManager接口，用于绕过验证，不用修改里面的方法
        X509TrustManager trustManager = new X509TrustManager() {
            @Override
            public void checkClientTrusted(
                    java.security.cert.X509Certificate[] paramArrayOfX509Certificate,
                    String paramString) throws CertificateException {
            }

            @Override
            public void checkServerTrusted(
                    java.security.cert.X509Certificate[] paramArrayOfX509Certificate,
                    String paramString) throws CertificateException {
            }

            @Override
            public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                return null;
            }
        };

        sc.init(null, new TrustManager[]{trustManager}, null);
        return sc;
    }


    public CookieStore cookieStore = new BasicCookieStore();

    private CloseableHttpClient getHttpClientSSL() throws KeyManagementException, NoSuchAlgorithmException {
        SSLContext sslcontext = createIgnoreVerifySSL();

        Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
                .register("http", PlainConnectionSocketFactory.INSTANCE)
                .register("https", new SSLConnectionSocketFactory(sslcontext))
                .build();
        PoolingHttpClientConnectionManager connManager = new PoolingHttpClientConnectionManager(socketFactoryRegistry);
        HttpClients.custom().setConnectionManager(connManager);

        CloseableHttpClient client = HttpClients.custom()
                .setConnectionManager(connManager)
                .setDefaultCookieStore(cookieStore)
                .build();
        return client;
    }

    private String entityToString(HttpEntity entity) throws IOException {
        String result = null;
        if (entity != null) {
            long lenth = entity.getContentLength();
            if (lenth != -1 && lenth < 2048) {
                result = EntityUtils.toString(entity, "UTF-8");
            } else {
                InputStreamReader reader1 = new InputStreamReader(entity.getContent(), "UTF-8");
                CharArrayBuffer buffer = new CharArrayBuffer(2048);
                char[] tmp = new char[1024];
                int l;
                while ((l = reader1.read(tmp)) != -1) {
                    buffer.append(tmp, 0, l);
                }
                result = buffer.toString();
            }
        }
        return result;
    }

    /**
     * 创建1个会话id
     *
     * @return
     */
    public static String getSessionId() {
        UUID uuid = UUID.randomUUID();
        String str = uuid.toString();
        return str.substring(0, 8) + str.substring(9, 13) + str.substring(14, 18) + str.substring(19, 23) + str.substring(24);
    }

    private static Logger logger = LoggerFactory.getLogger(NodeUtil.class);

}
