
package weaver.interfaces.bnkq.action;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.engine.kq.biz.*;
import com.engine.kq.entity.KQOvertimeRulesDetailEntity;
import com.engine.kq.entity.WorkTimeEntity;
import com.engine.kq.timer.KQOvertimeCardBean;
import com.engine.kq.util.TimeRangeCalculator;
import com.engine.kq.wfset.bean.OvertimeBalanceTimeBean;
import com.engine.kq.wfset.bean.SplitBean;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import weaver.conn.RecordSet;
import weaver.formmode.setup.ModeRightInfo;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.hrm.resource.ResourceComInfo;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.Property;
import weaver.soa.workflow.request.RequestInfo;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * 固定加班时长写入加班台账
 */
public class GdJbUpdateAction implements Action {

    static String dateFormat = "yyyy-MM-dd";
    static SimpleDateFormat format = new SimpleDateFormat(dateFormat);

    public GdJbUpdateAction() {
    }

    @Override
    public String execute(RequestInfo info) {
        new BaseBean();
        BaseBean bb = new BaseBean();
        bb.writeLog("固定加班时长写入加班台账");
        RecordSet rs = new RecordSet();
        int userid = info.getRequestManager().getUser().getUID();
        String wfid = info.getWorkflowid();
        String reqid = info.getRequestid();
        String tablaName = info.getRequestManager().getBillTableName();
        try {
            //获取主表数据
            Map mainInfo = getMainInfo(info);
            //加班人
            String jbr = Util.null2String(mainInfo.get("jbr"));
            //开始日期
            String ksrq = Util.null2String(mainInfo.get("ksrq"));
            //结束日期
            String jsrq = Util.null2String(mainInfo.get("jsrq"));
            //加班方式
            String jblx = Util.null2String(mainInfo.get("jbxz"));
            //报销方式
            String bcfs = Util.null2String(mainInfo.get("bcfs"));
            //申请事由
            String sqsy = Util.null2String(mainInfo.get("sqsy"));
            //开始时间
            String kssj = Util.null2String(mainInfo.get("kssj"));
            //结束时间
            String jssj = Util.null2String(mainInfo.get("jssj"));

            /*获取考勤组的ID，因为考勤组有有效期，所以需要传入日期*/
            KQGroupMemberComInfo kqGroupMemberComInfo = new KQGroupMemberComInfo();
            String groupId = kqGroupMemberComInfo.getKQGroupId(jbr, ksrq);

            //管理人员12小时班制考勤组id
            String glrygroupid = bb.getPropValue("nbkq_main", "glrygroupid");
            bb.writeLog("获取当前人员groupId:" + groupId + "glrygroupid:" + glrygroupid);
            if (groupId.equals(glrygroupid)) {
                buildOvertime(jbr, ksrq, jsrq, "固定加班流程生成台账", jblx, bcfs, sqsy, ksrq, jsrq, kssj, jssj);
            }


        } catch (Exception e) {
            e.printStackTrace();
            bb.writeLog(e);
            bb.writeLog("固定加班时长写入加班台账error" + e.getMessage());
            info.getRequestManager().setMessageid("2000");
            info.getRequestManager().setMessagecontent("固定加班时长写入加班台账error");
            return Action.FAILURE_AND_CONTINUE;
        }
        return Action.SUCCESS;
    }

    /**
     * 生成加班数据
     *
     * @param resourceid
     * @param fromDate
     * @param toDate
     * @param eventtype
     */
    public void buildOvertime(String resourceid, String fromDate, String toDate, String eventtype, String jblx, String bxfs, String sqsy, String ksrq, String jsrq, String kssj, String jssj) {
        BaseBean bb = new BaseBean();
        RecordSet singleRs = new RecordSet();
        try {
            DateTimeFormatter dateFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

            KQTimesArrayComInfo kqTimesArrayComInfo = new KQTimesArrayComInfo();
            KQOverTimeRuleCalBiz kqOverTimeRuleCalBiz = new KQOverTimeRuleCalBiz();
            Map<String, Integer> changeTypeMap = Maps.newHashMap();
            Map<String, KQOvertimeRulesDetailEntity> overRulesDetailMap = Maps.newHashMap();
            Map<String, List<String[]>> restTimeMap = Maps.newHashMap();
            Map<String, Integer> computingModeMap = Maps.newHashMap();
            //先获取一些前提数据，加班規則和假期規則
            kqOverTimeRuleCalBiz.getOverTimeDataMap(resourceid, fromDate, toDate, dateFormatter, changeTypeMap, overRulesDetailMap, restTimeMap, computingModeMap);
            Map<String, Object> eventLogMap = Maps.newHashMap();
            eventLogMap.put("fromDate", fromDate);
            eventLogMap.put("toDate", toDate);
            eventLogMap.put("eventtype", eventtype);
            String logKey = "|key|" + resourceid + "_" + fromDate + "_" + toDate;
            bb.writeLog("buildOvertime|生成加班调休" + JSONObject.toJSONString(eventLogMap) + "logKey" + logKey);
            LocalDate localFromDate = LocalDate.parse(fromDate);
            LocalDate localToDate = LocalDate.parse(toDate);
            LocalDate preFromDate = localFromDate.minusDays(1);
            LocalDate nextToDate = localToDate.plusDays(1);
            //之前是考虑外部考勤数据导入跨天打卡，判断归属的问题，向前算一天，现在不管了，都是默认只处理当天的
            if (eventtype.indexOf("#flow,") > -1 || eventtype.indexOf("punchcard") > -1 || true) {
                //如果是正常走的加班流程，就是流程开始日期和结束日期，只有补卡，打卡，同步的时候才需要处理一下前一天和后一天的数据
                preFromDate = localFromDate;
                nextToDate = localToDate;
            }
            long betweenDays = nextToDate.toEpochDay() - preFromDate.toEpochDay();
            for (int i = 0; i <= betweenDays; i++) {
                LocalDate curLocalDate = preFromDate.plusDays(i);
                String splitDate = curLocalDate.format(dateFormatter);
//        需审批，以打卡为准，取流程和打卡的交集
                doComputingMode4(resourceid, splitDate, dateFormatter, changeTypeMap, overRulesDetailMap, restTimeMap,
                        computingModeMap, kqTimesArrayComInfo, "", jblx, bxfs, sqsy, ksrq, jsrq, kssj, jssj, singleRs);
            }
        } catch (Exception e) {
            bb.writeLog("加班生成数据报错:KQOverTimeRuleCalBiz:");
        }
    }

    /**
     * 需审批，以打卡为准，取流程和打卡的交集
     *
     * @param resourceid
     * @param splitDate
     * @param dateFormatter
     * @param changeTypeMap
     * @param overRulesDetailMap
     * @param restTimeMap
     * @param computingModeMap
     * @param kqTimesArrayComInfo
     * @param main_uuid
     * @param singleRs
     */
    private void doComputingMode4(String resourceid, String splitDate,
                                  DateTimeFormatter dateFormatter, Map<String, Integer> changeTypeMap,
                                  Map<String, KQOvertimeRulesDetailEntity> overRulesDetailMap,
                                  Map<String, List<String[]>> restTimeMap, Map<String, Integer> computingModeMap,
                                  KQTimesArrayComInfo kqTimesArrayComInfo, String main_uuid, String jblx, String bxfs, String sqsy, String ksrq, String jsrq, String kssj, String jssj, RecordSet singleRs) throws Exception {
        String key = resourceid + "_" + splitDate;
        //加班日志记录类
        KQWorkTime kqWorkTime = new KQWorkTime();
        BaseBean bb = new BaseBean();
        WorkTimeEntity workTimeEntity = kqWorkTime.getWorkTime(resourceid, splitDate);
        Map<String, Object> workTimeEntityLogMap = Maps.newHashMap();
        workTimeEntityLogMap.put("resourceid", resourceid);
        workTimeEntityLogMap.put("splitDate", splitDate);
        workTimeEntityLogMap.put("workTimeEntity", workTimeEntity);
        Map<String, Object> overtimeLogMap = Maps.newLinkedHashMap();
        Map<String, Object> eventMap = Maps.newLinkedHashMap();
        Map<String, KQOvertimeCardBean> lsCheckInfoMaps = Maps.newLinkedHashMap();
        KQOverTimeRuleCalBiz kqOverTimeRuleCalBiz = new KQOverTimeRuleCalBiz();
        //获取加班打卡数据
        kqOverTimeRuleCalBiz.getOverTimeCardDataMap(resourceid, splitDate, splitDate, dateFormatter, kqTimesArrayComInfo, overRulesDetailMap, changeTypeMap, lsCheckInfoMaps, eventMap);
        bb.writeLog("获取打卡数据结束lsCheckInfoMaps:" + JSONObject.toJSONString(lsCheckInfoMaps));
        if (lsCheckInfoMaps.isEmpty()) {
            bb.writeLog("对应的加班流程数据key" + key);
            return;
        }
        Iterator<Map.Entry<String, KQOvertimeCardBean>> iterator = lsCheckInfoMaps.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, KQOvertimeCardBean> next = iterator.next();
            String mapKey = next.getKey();
            KQOvertimeCardBean kqOvertimeCardBean = next.getValue();
            String[] mapKeys = mapKey.split("_");
            if (mapKeys.length != 2) {
                continue;
            }
            String realSplitDate = mapKeys[1];
            Map<String, List<SplitBean>> splitBeanMaps = Maps.newHashMap();
            //获取加班流程数据
            getOverTimeFlowData(resourceid, realSplitDate, realSplitDate, splitBeanMaps, dateFormatter);
            String change_key = realSplitDate + "_" + resourceid;
            int changeType = Util.getIntValue("" + changeTypeMap.get(change_key), -1);
            String changeType_key = realSplitDate + "_" + changeType;
            String changetypeName = 1 == changeType ? "节假日" : (2 == changeType ? "工作日" : (3 == changeType ? "休息日" : "异常"));
            String changetypeLogInfo = change_key + "|changeType|" + changeType + "|" + changetypeName + "changeType_key" + changeType_key;
            bb.writeLog("changetypeLogInfo:" + changetypeLogInfo);
            KQOvertimeRulesDetailEntity kqOvertimeRulesDetailEntity = overRulesDetailMap.get(changeType_key);
            if (kqOvertimeCardBean != null) {

                int[] initArrays = kqTimesArrayComInfo.getInitArr();
                List<Map<String, String>> hasOverTime4SignList = Lists.newArrayList();
                getHasOverTimeData(resourceid, realSplitDate, hasOverTime4SignList);
                bb.writeLog("kqOvertimeCardBean:" + JSONObject.toJSONString(kqOvertimeCardBean));
                bb.writeLog("realSplitDate:" + realSplitDate);
                bb.writeLog("hasOverTime4SignList:" + JSONObject.toJSONString(hasOverTime4SignList));
                Map<String, String> signinoffMap = kqOverTimeRuleCalBiz.buildOvertimeCard(kqOvertimeCardBean, resourceid, realSplitDate, kqTimesArrayComInfo, restTimeMap, changeType_key, initArrays, hasOverTime4SignList,
                        overRulesDetailMap, true, overtimeLogMap);
                bb.writeLog("signinoffMap:" + JSONObject.toJSONString(signinoffMap));
                String signinTime = Util.null2String(signinoffMap.get("signinTime"));
                String signoutTime = Util.null2String(signinoffMap.get("signoutTime"));
                String signinDate = Util.null2String(signinoffMap.get("signinDate"));
                String signoutDate = Util.null2String(signinoffMap.get("signoutDate"));
                int signinTimeIndex = kqTimesArrayComInfo.getArrayindexByTimes(signinTime);
                int signoutTimeIndex = kqTimesArrayComInfo.getArrayindexByTimes(signoutTime);
                Integer check = Integer.valueOf(signinTime.substring(0, 2));
                if (check > 24) {
                    bb.writeLog("不插入台账");
                } else {
                    if (signinTimeIndex < signoutTimeIndex) {
                        int across_mins = 0;
                        int kssjIndex = kqTimesArrayComInfo.getArrayindexByTimes(kssj + ":00");
                        int jssjIndex = kqTimesArrayComInfo.getArrayindexByTimes(jssj + ":00");
                        int checkIndex = kqTimesArrayComInfo.getArrayindexByTimes("20:00:00");
                        int check2Index = kqTimesArrayComInfo.getArrayindexByTimes("06:30:00");
                        String endTime = "";
                        if (signoutTimeIndex < jssjIndex) {
                            endTime = signoutTime;
                        } else {
                            endTime = jssj + ":00";
                        }
                        bb.writeLog("endTime" + endTime);
                        if (jssjIndex < checkIndex) {
                            //夜班
                            //白班
                            String kssjNew = "";
                            if (kssjIndex < check2Index) {
                                kssjNew = "06:30";
                            } else {
                                kssjNew = kssj;
                            }
                            across_mins = divide(String.valueOf(TimeUtil.timeInterval(ksrq + " " + kssjNew + ":00", jsrq + " " + endTime)), "60").intValue();
                        } else {
                            //白班
                            String kssjNew = "";
                            if (kssjIndex < checkIndex) {
                                kssjNew = "20:00";
                            } else {
                                kssjNew = kssj;
                            }

                            across_mins = divide(String.valueOf(TimeUtil.timeInterval(ksrq + " " + kssjNew + ":00", jsrq + " " + endTime)), "60").intValue();
                        }


                        bb.writeLog("最终生成的加班分钟数" + across_mins);
                        /*获取考勤组的ID，因为考勤组有有效期，所以需要传入日期*/
                        KQGroupMemberComInfo kqGroupMemberComInfo = new KQGroupMemberComInfo();
                        String groupId = kqGroupMemberComInfo.getKQGroupId(resourceid, realSplitDate);
                        bb.writeLog("获取当前人员groupId:" + groupId + "across_mins:" + across_mins);
                        //管理人员12小时班制考勤组id
                        String glrygroupid = bb.getPropValue("nbkq_main", "glrygroupid");
                        Integer bxszh = changeBxs(across_mins);
                        Integer yxszh = changeyxs(across_mins);
                        String searIdFrom = workTimeEntity.getSerialId();
                        bb.writeLog("半小时计提时长bxszh:" + bxszh + "yxszh:" + yxszh + "searIdFrom:" + searIdFrom);
                        bb.writeLog("jblx:" + jblx);
                        if (groupId.equals(glrygroupid)) {
                            bb.writeLog("是管理人员12小时");
                            //是管理人员12小时班
                            if ("0".equals(jblx)) {
                                //临时去一线
                                //判断是否小于0.5
                                if (across_mins < 30) {
                                    bb.writeLog("小于半小时计0");
                                    across_mins = 0;
                                } else {
                                    bb.writeLog("半小时计提");
                                    across_mins = bxszh;
                                }
                            } else {
                                //管理人员的12小时班制加班规则是加班最小单位是1小时，以1小时为递增计算单位,例如1、2、3
                                //判断是否小于1
                                if (across_mins < 60) {
                                    bb.writeLog("小于1小时计0");
                                    across_mins = 0;
                                } else {
                                    bb.writeLog("1小时计提");
                                    across_mins = yxszh;
                                }
                            }
                        }

                        bb.writeLog("预计插入时长:" + across_mins);

                        signinTime = kqTimesArrayComInfo.turn48to24Time(signinTime);
                        signoutTime = kqTimesArrayComInfo.turn48to24Time(signoutTime);

                        if (changeType == 2) {
                            deleteJbtz(resourceid, "0", realSplitDate);
                            insertJbtz(resourceid, String.valueOf(across_mins), resourceid, jblx, bxfs,
                                    signinDate + " " + signinTime, signoutDate + " " + signoutTime, "0",
                                    String.valueOf(changeType), ksrq, jsrq, sqsy, kssj, jssj, realSplitDate, searIdFrom, singleRs);
                        }
                    } else {

                        bb.writeLog("打卡数据异常");
                    }
                }

            } else {
                bb.writeLog("打卡数据KQOvertimeCardBean为null");
            }
        }
    }

    /**
     * 获取两个日期之间所有的日期
     *
     * @param date1
     * @param date2
     * @return
     */
    public static ArrayList days(String date1, String date2) {
        ArrayList L = new ArrayList();
        if (date1.equals(date2)) {
            System.out.println("两个日期相等!");
            return L;
        }

        String tmp;
        if (date1.compareTo(date2) > 0) { // 确保 date1的日期不晚于date2
            tmp = date1;
            date1 = date2;
            date2 = tmp;
        }

        tmp = format.format(str2Date(date1).getTime() + 3600 * 24 * 1000);

        int num = 0;
        while (tmp.compareTo(date2) < 0) {
            L.add(tmp);
            num++;
            tmp = format.format(str2Date(tmp).getTime() + 3600 * 24 * 1000);
        }

        if (num == 0) {
            System.out.println("两个日期相邻!");
        }
        return L;
    }


    private static Date str2Date(String str) {
        if (str == null) {
            return null;
        }

        try {
            return format.parse(str);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * 获取主表数据
     *
     * @param requestInfo
     * @return
     */
    private static Map getMainInfo(RequestInfo requestInfo) {
        Map map = new HashMap();
        Property[] property = requestInfo.getMainTableInfo().getProperty();
        for (int i = 0; i < property.length; i++) {
            map.put(property[i].getName().toLowerCase(), Util.null2String(property[i].getValue()));
        }
        return map;
    }

    /**
     * 提供精确的乘法运算。
     *
     * @param v1 被加数
     * @param v2 加数
     * @return 两个参数的和
     */
    public static Double multiply(String v1, String v2) {
        BigDecimal b1 = new BigDecimal(v1);
        BigDecimal b2 = new BigDecimal(v2);
        return b1.multiply(b2).doubleValue();
    }

    /**
     * 插入请假拆分表
     *
     * @param requestid
     * @param userid
     * @param fromDate
     * @param MaxHours
     */
    private static void insertLeave(String requestid, Integer userid, String fromDate, Double MaxHours, String durationrule, String newleavetype, String subcompany, String department, String toDate, String wfid, String tablename) {
        BaseBean bb = new BaseBean();
        //插入加班时间
        RecordSet rsInsert = new RecordSet();
        String sql = "insert into kq_flow_split_evection (requestid,resourceid,fromdate,fromtime,todate,totime,duration,durationrule,belongdate,newleavetype,subcompanyid,departmentid,workflowid,tablenamedb) "
                + " values(?,?,?,?,?,?,?,?,?,?,?,?,?,?) ";
        rsInsert.executeUpdate(sql, requestid, userid, fromDate, "08:00", toDate, "17:00", MaxHours, durationrule, fromDate, newleavetype, subcompany, department, wfid, tablename);
        bb.writeLog("insertSql:" + sql);
    }


    /**
     * 半小时转换
     *
     * @param value
     * @return
     */
    public static Integer changeBxs(Integer value) {
        return (value / 30) * 30;
    }

    /**
     * 1小时转换
     *
     * @param value
     * @return
     */
    public static Integer changeyxs(Integer value) {
        return (value / 60) * 60;
    }

    /**
     * 先删除台账
     *
     * @param jbr
     * @param ly
     * @param date
     */
    public static void deleteJbtz(String jbr, String ly, String date) {
        RecordSet rs = new RecordSet();
        String sql = "delete from uf_jbtz where jbr='" + jbr + "' and ly =" + ly + " and gzrq = '" + date + "' ";
        rs.executeUpdate(sql);
    }


    /**
     * 加班台账数据写入
     */
    public static void insertJbtz(String jbr, String gjsc, String sqr, String jbxz, String bcfs, String zzdksj, String zwdkrq, String ly, String jblx, String ksrq, String jsrq, String sqsy, String kssj, String jssj, String szrq, String searIdFrom, RecordSet singleRs) {
        RecordSet rs = new RecordSet();
        String idNew = "1";
        int modeid = getFormModeIdByCubeName("uf_jbtz");
        SimpleDateFormat sdfDate = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdfTime = new SimpleDateFormat("HH:mm:ss");
        String modedatacreatedate = sdfDate.format(new Date());
        String modedatacreatetime = sdfTime.format(new Date());
        try {
            ResourceComInfo resourceComInfo = new ResourceComInfo();
            String bm = resourceComInfo.getDepartmentID(jbr);
            String szbm = resourceComInfo.getDepartmentID(sqr);
            String sjh = resourceComInfo.getTelephone(jbr);
            String gh = resourceComInfo.getWorkcode(jbr);
            String zw = resourceComInfo.getJobActivityDesc(jbr);
            Double sc = divide(gjsc, "60");
            String jblxNew = "";
            if ("1".equals(jblx)) {
                jblxNew = "2";
            } else if ("2".equals(jblx)) {
                jblxNew = "0";
            } else if ("3".equals(jblx)) {
                jblxNew = "1";
            }
            int weekDay = TimeUtil.dateWeekday(ksrq);
            if (weekDay == 0 || weekDay == 6) {
                jblxNew = "1";
            }
            //如果是休班次，加班类型为休息日加班
            if (isRestSerial(searIdFrom, rs)) {
                jblxNew = "1";
                sc = recalculateOverTime(sc, searIdFrom, ksrq, kssj, jsrq, jssj, rs, singleRs);
            }


            String sqlMode = "insert into uf_jbtz (jbr,gjsc,bcfs,jbxz,sqr,szbm,sjh,zzdksj,zwdkrq,ly,gh,bm,zw,jblx,ksrq,jsrq,sqsy,kssj,jssj,gzrq,FORMMODEID,MODEDATACREATER,MODEDATACREATERTYPE,MODEDATACREATEDATE,MODEDATACREATETIME) \n" +
                    "values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
            rs.executeUpdate(sqlMode, jbr, sc, bcfs, jbxz, sqr, szbm, sjh, zzdksj, zwdkrq, ly, gh, bm, zw, jblxNew, ksrq, jsrq, sqsy, kssj, jssj, szrq, modeid, jbr, "0", modedatacreatedate, modedatacreatetime);

            RecordSet findNew = new RecordSet();
            String sqlFindnew = "select Id from uf_jbtz where jbr = '" + jbr + "' and zzdksj = '" + zzdksj + "' and zwdkrq = '" + zwdkrq + "'  order by" +
                    " modedatacreatedate desc,modedatacreatetime desc";
            findNew.execute(sqlFindnew);

            if (findNew.next()) {
                idNew = Util.null2String(findNew.getString("id"));
            }

            // 权限重构
            int dataId = Integer.valueOf(idNew);
            ModeRightInfo modeRightInfo = new ModeRightInfo();
            modeRightInfo.editModeDataShare(Integer.valueOf(jbr), modeid, dataId);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static Double recalculateOverTime(Double sc, String serialId, String ksrq, String kssj, String jsrq, String jssj, RecordSet rs, RecordSet singleRs) {
        BaseBean bb = new BaseBean();
        try {
            boolean isAcross = false;
            BigDecimal workTimeTotal = new BigDecimal(TimeUtil.timeInterval(ksrq + " " + kssj + ":00", jsrq + " " + jssj + ":00"));
            BigDecimal restTimeCount = new BigDecimal(0);
            if (ksrq.compareTo(jsrq) > 0) {
                isAcross = true;
            }
            String restStartSql = "select * from kq_ShiftRestTimeSections where serialid=? and resttype='start'";
            rs.executeQuery(restStartSql, serialId);
            while (rs.next()) {
                //休息开始时间
                String startTime = rs.getString("time");
                String record1 = rs.getString("record1");
                //休息结束时间
                String restEndSql = "select * from kq_ShiftRestTimeSections where record1=? and resttype='end'";
                singleRs.executeQuery(restEndSql, record1);
                singleRs.next();
                String endTime = singleRs.getString("time");
                if (isAcross) {
                    double restTime1 = TimeRangeCalculator.getRestTime(kssj, "23:59", startTime, endTime);
                    double restTime2 = TimeRangeCalculator.getRestTime("00:00", jssj, startTime, endTime);
                    restTimeCount = restTimeCount.add(BigDecimal.valueOf(restTime1));
                    restTimeCount = restTimeCount.add(BigDecimal.valueOf(restTime2));

                } else {
                    double restTime = TimeRangeCalculator.getRestTime(kssj, jssj, startTime, endTime);
                    restTimeCount = restTimeCount.add(BigDecimal.valueOf(restTime));
                }
            }
            sc = divideRoundModeDown(workTimeTotal.subtract(restTimeCount).toString(), "3600");
            //加班最小单位是2小时
            //一小时和两小时起休班次id
            String twoStartSerId = bb.getPropValue("nbkq_main", "twoStartSerId");
            if (Arrays.asList(twoStartSerId.split(",")).contains(serialId) && sc < 2) {
                sc = 0.0;
            }
            return sc;
        } catch (Exception e) {
            bb.writeLog("recalculateOverTime-e-" + e.getMessage());
            return sc;
        }
    }

    private static boolean isRestSerial(String serialId, RecordSet rs) {
        try {
            String querySql = "select rest_shift from kq_ShiftManagement where id=?";
            rs.executeQuery(querySql, serialId);
            if (rs.next()) {
                return rs.getInt("rest_shift") == 1;
            }
        } catch (Exception e) {
            return false;
        }
        return false;
    }

    /**
     * 提供精确的除法运算。
     *
     * @param v1 被加数
     * @param v2 加数
     * @return 两个参数的和
     */
    public static Double divide(String v1, String v2) {
        BigDecimal b1 = new BigDecimal(v1);
        BigDecimal b2 = new BigDecimal(v2);
        return b1.divide(b2, 2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    public static Double divideRoundModeDown(String v1, String v2) {
        BigDecimal b1 = new BigDecimal(v1);
        BigDecimal b2 = new BigDecimal(v2);
        BigDecimal divide = b1.divide(b2, 2, BigDecimal.ROUND_HALF_UP);
        return divide.setScale(0, RoundingMode.DOWN).doubleValue();
    }

    /**
     * 找到归属日期当天已生成的打卡时长
     *
     * @return
     */
    public static Double findSc(String jbr, String gsrq) {
        RecordSet rs = new RecordSet();
        String sql = "select gjsc from uf_jbtz where jbr = " + jbr + " and gzrq = '" + gsrq + "'";
        rs.executeQuery(sql);
        String gjsc = "";
        if (rs.next()) {
            gjsc = Util.null2String(rs.getString("gjsc"));
        }
        return multiply(gjsc, "60");

    }

    /**
     * 根据建模表名获取formModeId
     *
     * @param cubeName 建模表明
     * @return int formModeId
     */
    private static int getFormModeIdByCubeName(String cubeName) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select t1.id from modeinfo t1\n" +
                " left join workflow_bill t2\n" +
                " on t1.formid=t2.id\n" +
                " where t2.tablename='" + cubeName + "'");
        rs.next();
        return rs.getInt("id");
    }

    /**
     * 获取加班流程数据
     *
     * @param resourceid
     * @param fromDate
     * @param toDate
     * @param splitBeanMaps
     * @param dateFormatter
     */
    private void getOverTimeFlowData(String resourceid, String fromDate, String toDate,
                                     Map<String, List<SplitBean>> splitBeanMaps,
                                     DateTimeFormatter dateFormatter) {
        LocalDate localFromDate = LocalDate.parse(fromDate);
        LocalDate preFromDate = localFromDate.minusDays(1);
        LocalDate localToDate = LocalDate.parse(toDate);
        long betweenDays = localToDate.toEpochDay() - preFromDate.toEpochDay();
        for (int k = 0; k <= betweenDays; k++) {
            LocalDate curLocalDate = preFromDate.plusDays(k);
            String splitDate = curLocalDate.format(dateFormatter);
            String key = resourceid + "_" + splitDate;
            String order_sql = " order by belongdate,fromtime ";
            KQFlowDataBiz kqFlowDataBiz = new KQFlowDataBiz.FlowDataParamBuilder().belongDateParam(splitDate).resourceidParam(resourceid).orderby_sqlParam(order_sql).build();
            Map<String, Object> flowMaps = Maps.newHashMap();
            List<SplitBean> splitBeans = kqFlowDataBiz.getOverTimeData(flowMaps);
            if (!splitBeans.isEmpty()) {
                splitBeanMaps.put(key, splitBeans);
            }
        }
    }

    /**
     * 获取已经生成过的加班数据
     *
     * @param resourceid
     * @param belongdate
     * @param hasOverTimeList
     */
    private void getHasOverTimeData(String resourceid, String belongdate, List<Map<String, String>> hasOverTimeList) {
        RecordSet rs = new RecordSet();

        String sql = " select * from kq_flow_overtime where resourceid = ? and belongdate = ? ";
        rs.executeQuery(sql, resourceid, belongdate);
        while (rs.next()) {
            String requestid = rs.getString("requestid");
            String fromdate = rs.getString("fromdate");
            String fromtime = rs.getString("fromtime");
            String todate = rs.getString("todate");
            String totime = rs.getString("totime");
            String duration_min = Util.null2String(rs.getString("duration_min"));
            String flow_dataid = Util.null2String(rs.getString("flow_dataid"));
            String ori_belongdate = Util.null2String(rs.getString("ori_belongdate"));
            //流程+打卡的时候，存的对应的流程数据
            String fromdate_flow = rs.getString("fromdatedb");
            String fromtime_flow = rs.getString("fromtimedb");
            String todate_flow = rs.getString("todatedb");
            String totime_flow = rs.getString("totimedb");
            Map<String, String> hasOverTimeMap = Maps.newHashMap();
            hasOverTimeMap.put("resourceid", resourceid);
            hasOverTimeMap.put("belongdate", belongdate);
            hasOverTimeMap.put("requestid", requestid);
            hasOverTimeMap.put("fromdate", fromdate);
            hasOverTimeMap.put("fromtime", fromtime);
            hasOverTimeMap.put("todate", todate);
            hasOverTimeMap.put("totime", totime);
            hasOverTimeMap.put("fromdate_flow", fromdate_flow);
            hasOverTimeMap.put("fromtime_flow", fromtime_flow);
            hasOverTimeMap.put("todate_flow", todate_flow);
            hasOverTimeMap.put("totime_flow", totime_flow);
            hasOverTimeMap.put("duration_min", duration_min);
            hasOverTimeMap.put("flow_dataid", flow_dataid);
            hasOverTimeMap.put("ori_belongdate", ori_belongdate);

            hasOverTimeList.add(hasOverTimeMap);
        }
    }
}
